@@ -5,6 +5,7 @@ import os
 import json
 import csv
 import gzip
+from collections import defaultdict
 
 from graphio.helper import chunks, create_single_index, create_composite_index
 from graphio import defaults
@@ -23,7 +24,7 @@ class NodeSet:
     Container for a set of Nodes with the same labels and the same properties that define uniqueness.
     """
 
-    def __init__(self, labels, merge_keys=None, batch_size=None, default_props=None, preserve=None, append_props=None):
+    def __init__(self, labels, merge_keys=None, batch_size=None, default_props=None, preserve=None, append_props=None, indexed=False):
         """
 
         :param labels: The labels for the nodes in this NodeSet.
@@ -38,6 +39,7 @@ class NodeSet:
         self.default_props = default_props
         self.preserve = preserve
         self.append_props = append_props
+        self.indexed = indexed
 
         self.combined = '_'.join(sorted(self.labels)) + '_' + '_'.join(sorted(self.merge_keys))
         self.uuid = str(uuid4())
@@ -48,10 +50,22 @@ class NodeSet:
             self.batch_size = defaults.BATCHSIZE
 
         self.nodes = []
+        # a node index with merge_key_id -> [positions in nodes list]
+        # this works for both unique and non-unique settings
+        self.node_index = defaultdict(list)
 
     def __str__(self):
         return f"<NodeSet ({self.labels}; {self.merge_keys})>"
 
+    def _merge_key_id(self, node_dict: dict) -> tuple:
+        """
+        Create a FrozenSet from an ordered list of the merge_key properties for a node.
+
+        :param node_dict: A node dict.
+        :return:
+        """
+        return tuple([node_dict[key] for key in self.merge_keys])
+
     def add_node(self, properties):
         """
         Create a node in this NodeSet.
@@ -66,10 +80,33 @@ class NodeSet:
 
         self.nodes.append(node_props)
 
+        if self.indexed:
+            self.node_index[self._merge_key_id(properties)].append(len(self.nodes) - 1)
+
     def add_nodes(self, list_of_properties):
         for properties in list_of_properties:
             self.add_node(properties)
 
+    def update_node(self, properties: dict):
+        """
+        Update an existing node by overwriting all properties.
+
+        Note that this requires `NodeSet(..., indexed=True)` which is not the default!
+
+        :param properties: Node property dictionary.
+        """
+        if not self.indexed:
+            raise TypeError("Update only works on an indexed NodeSet.")
+
+        node_merge_key_id = self._merge_key_id(properties)
+        if node_merge_key_id in self.node_index:
+            # this function should work for single and multiple nodes
+            for node_list_index in self.node_index[node_merge_key_id]:
+                self.nodes[node_list_index].update(properties)
+        # if the node does not exist it is created
+        else:
+            self.add_node(properties)
+
     def add_unique(self, properties):
         """
         Add a node to this NodeSet only if a node with the same `merge_keys` does not exist yet.
