@@ -281,3 +281,39 @@ class TestRelationshipSetSerialize:
                 assert reloaded_relset.end_node_properties == test_rs.end_node_properties
                 assert reloaded_relset.relationships == test_rs.relationships
                 assert len(reloaded_relset.relationships) == len(test_rs.relationships)
+
+
+class TestRelationshipSetCSVandJSON:
+    """
+    Test functionality around creating/reading CSV files and associated JSON metadata files.
+    """
+    def test_read_from_files(self, root_dir, clear_graph, graph):
+
+        files_path = os.path.join(root_dir, "test", "files")
+
+        # load nodes first (taken from NodeSet test)
+        nodes_json_file_path = os.path.join(files_path, 'nodes_csv_json.json')
+        nodes_csv_file_path = os.path.join(files_path, 'nodes_csv_json.csv')
+
+        ns = NodeSet.from_csv_json_set(nodes_csv_file_path, nodes_json_file_path)
+        ns.merge(graph)
+        assert list(graph.run("MATCH (n:Test) RETURN count(n)"))[0][0] == 2
+
+        # create relset and load relationships
+        json_file_path = os.path.join(files_path, 'rels_csv_json.json')
+        csv_file_path = os.path.join(files_path, 'rels_csv_json.csv')
+
+        assert os.path.exists(json_file_path)
+        assert os.path.exists(csv_file_path)
+
+        rs = RelationshipSet.from_csv_json_set(csv_file_path, json_file_path)
+
+        assert rs.start_node_labels == ['Test']
+        assert rs.end_node_labels == ['Test']
+        assert rs.start_node_properties == ['test_id']
+        assert rs.end_node_properties == ['test_id']
+        assert rs.rel_type == 'RELATIONSHIP'
+
+        rs.merge(graph)
+
+        assert list(graph.run("MATCH (:Test)-[r:RELATIONSHIP]->(:Test) RETURN count(r)"))[0][0] == 1
